from coverage_reporter.collectors.base import BaseCollector
from coverage_reporter.plugins import Option

class CoveragePyCollector(BaseCollector):
    """
    Coverage collector for the coverage.py tool.
    """
    name = 'coverage'

    options = [ Option('coverage', 'boolean', 
                       help='Enables loading of coverage information from coverage.py') ]

    def should_cover(self, path):
        return path.endswith('.py')

    def get_all_lines_from_path(self, path):
        from coverage.parser import CodeParser
        parser = CodeParser(filename=path)
        statements, excluded = parser.parse_source()
        return statements

    def collect_covered_lines(self):
       from coverage.data import CoverageData
       data = CoverageData()
       data.read_file('.coverage')
       return data.lines



from coverage_reporter.errors import PluginError, ConfigError

__all__ = ['Plugin', 'Option']

class Plugin(object):
    """
    (Optional) Base class for plugins.  Provides some useful behaviors.

    Among the most useful is options support.  A plugin can specify a number of options
    in the options class variable.  The options should be instances of coverage_reporter.plugins.Option.
    All such options will get their value from configuration files (if present), but their value will
    be overridden by anything specified by a command-line flag.

    The option value will be stored in an attribute of the plugin of the same name as the option.

    By default, a plugin will be assumed to be enabled if the value of the first option passed in is
    set.  This can be overridden by defining the "enabled" method.

    Plugins can currently provide 5 hooks:
        * collect(path_list, path_filter) - returns a data.CoverageData object with active lines and covered lines.
        * filter(coverage_data) - removes items from a data.CoverageData object.
        * cover_path(path) - returns True if this line should be considered for coverage
        * report_path(path, coverage_data) - returns True if this line should be reported on
        * report(coverage_data, path_list) - performs some sort of operation on the final reporting data.

    Examples of all types are shown in collectors, filters, and reporters subdirs.
    """
    options = []

    def __init__(self, cfg):
        """
        Used for actions performed upon loading but before knowing whether the plugin is enabled or not. 
        """
        self.cfg = cfg

    def add_options(self, parser):
        for option in self.options:
            option.add_option(parser)

    def parse_options(self, options):
        for option in self.options:
            value = option.get_value(options, self.cfg)
            setattr(self, option.name, value)

    def enabled(self):
        if self.options:
            # we make the handy (documented) assumption that if the first option 
            # is set then this plugin is being used.
            return getattr(self, self.options[0].name, False)
        return True

    def initialize(self):
        pass


class Option(object):
    """
    Option wrapper for coverage_reporter plugins.

    This class allows you define an option once, in a way that is shared between config files and optparse flags.

    Option takes all parameters that the normal parser.add_option command would, except "action", "dest", and the flag name(s).
    These are all inferred from the name and option_type.

    The option_type possibilites are currently:
        int
        string
        list
        boolean

    These are passed in as the second parameter to the Option class.
    """

    def __init__(self, name, option_type, default=None, **option_kwargs):
        self.name = name
        self.option_type = option_type
        self.default = default
        self.option_kwargs = option_kwargs

    def add_option(self, parser):
        if self.option_type in ('int', 'string'):
            action = 'store'
        elif self.option_type in ('boolean',):
            action = 'store_true'
        elif self.option_type in ('list',):
            action = 'append'
        else:
            raise ConfigError('Invalid option type %r' % (self.option_type,))
        parser.add_option('--' + self.name.replace('_', '-'), dest=self.name, action=action, **self.option_kwargs)

    def get_value(self, options, cfg):
        """
        Returns value for this option from either cfg object or optparse option list, preferring the option list.
        """
        if self.option_type in ('string', 'int', 'boolean'):
            value = getattr(options, self.name, None)
            if value is None:
                value = cfg.get(self.name, None)
            if value is None:
                value = self.default
            if self.option_type == 'int' and value is not None:
                value = int(value)
            elif self.option_type == 'boolean' and value is not None:
                if value.lower() in ('true', '1'):
                    return True
                elif value.lower() in ('false', '0'):
                    return False
                else:
                    raise ConfigError('Invalid value for option %r: %r' % (self.name, value))
            return value
        elif self.option_type == 'list':
            value = getattr(options, self.name, None)
            if value is None:
                value = cfg.get_list(self.name, [])
            return value

class PluginManager(object):
    """
    Class that takes care of loading and managing plugins.
    """
    def __init__(self):
        self.plugins = []
        self.collectors = []
        self.coverage_filters = []
        self.filters = []
        self.reporter_filters = []
        self.reporters = []

    def call_method(self, method_name, *args, **kwargs):
        for plugin in self.plugins:
            method = getattr(plugin, method_name, None)
            if not method:
                continue
            method(*args, **kwargs)

    def load_plugins(self, cfg_factory, class_strings):
        classes = self.load_classes(class_strings)
        for class_ in classes:
            self.validate_plugin(class_)
            plugin = class_(cfg_factory(class_.name))
            self.plugins.append(plugin)
            if hasattr(plugin, 'collect'):
                self.collectors.append(plugin)
            if hasattr(class_, 'filter'):
                self.filters.append(plugin)

            if hasattr(class_, 'report_path'):
                self.reporter_filters.append(plugin)
            if hasattr(class_, 'cover_path'):
                self.coverage_filters.append(plugin)
            if hasattr(class_, 'report'):
                self.reporters.append(plugin)

    def validate_plugin(self, plugin):
        required_attributes = ['enabled', 'name']
        for attr in required_attributes:
            if not hasattr(plugin, attr):
                raise PluginError('Plugin %r is missing %r attribute' % (plugin, attr))

    def get_collectors(self):
        return [ x for x in self.collectors if x.enabled() ]

    def get_filters(self):
        return [ x for x in self.filters if x.enabled() ]

    def get_coverage_filter(self):
        return Filter(*[ x.cover_path for x in self.coverage_filters if x.enabled() ])

    def get_reporter_filter(self):
        return Filter(*[ x.report_path for x in self.reporter_filters if x.enabled() ])

    def get_reporters(self):
        return [ x for x in self.reporters if x.enabled() ]

    def load_classes(self, class_strings):
        classes = []
        for class_string in class_strings:
            module_path, class_name = class_string.rsplit('.', 1)
            module = __import__(module_path, fromlist=True)
            class_ = getattr(module, class_name)
            classes.append(class_)
        return classes


class Filter(object):
    """
    Tracks several functions, returning True only if all of the referenced 
    functions return True.
    """

    def __init__(self, *filter_fns):
        self.filter_fns = filter_fns

    def filter_all(self, iterable, *args, **kwargs):
        for item in iterable:
            for filter_fn in self.filter_fns:
                if not filter_fn(item, *args, **kwargs):
                    break
            else:
                # all filters passed for item.
                yield item

    def filter(self, *args, **kwargs):
        for filter in self.filter_fns:
            if not filter(*args, **kwargs):
                return False
        return True



import os
import types

from coverage_reporter.collectors.base import BaseCollector
from coverage_reporter.plugins import Option


class FigleafCollector(BaseCollector):
    """
    Coverage collector for figleaf's coverage info.
    """
    name = 'figleaf'
    options = [Option('figleaf', 'boolean', 
                      help='Enables loading of coverage information from figleaf') ]

    def should_cover(self, path):
        return path.endswith('.py')
    
    def get_all_lines_from_path(self, path):
        if '<doctest' in path:
            return set()
        if not os.path.exists(path):
            return set()
        import figleaf
        try:
            return figleaf.get_lines(open(path))
        except:
            return set()

    def collect_covered_lines(self):
        import figleaf
        return figleaf.read_coverage('.figleaf')



from coverage_reporter.collectors import util
from coverage_reporter import data
from coverage_reporter.plugins import Plugin

class BaseCollector(Plugin):
    """
    Base class for collector types.  Splits behavior into two sections:
        - collecting information about what lines were covered
        - collecting list of all possible lines that _could_ have been covered.

    """

    def collect(self, path_list, path_filter):
        coverage_data = data.CoverageData()
        covered_lines = self.collect_covered_lines()
        coverage_data.update_coverage(covered_lines)
        all_lines = self.collect_all_lines(path_list, covered_lines, path_filter)
        coverage_data.update_lines(all_lines)
        return coverage_data

    def should_cover(self, path):
        """
        Returns True if this coverage tool can handle covering this path.
        """
        return False

    def collect_covered_lines(self):
        """
        Returns dictionary of { path : [ line, line ] } that describes every actually covered line.
        """
        return {}

    def collect_all_lines(self, path_list, covered_lines, path_filter):
        """
        Returns dictionary of { path : [ line, line ] } that describes every possible coverable line.
        """
        line_dict = {}
        paths = list(util.filter_paths(path_list, covered_lines, path_filter))
        for path in util.filter_paths(path_list, covered_lines, path_filter):
            if self.should_cover(path):
                line_dict[path] = self.get_all_lines_from_path(path)
        return line_dict

    def get_total_lines_from_path(self, path):
        """
        Returns a list of every possible coverable line in <path>.
        """
        return []



from coverage_reporter.plugins import Plugin, Option

class MinimumMissingFilter(Plugin):
    """
    Simple filter that allows you to only display files that are missing large amounts of coverage.
    """
    name = 'missing'
    options = [ Option('minimum_missing', 'int', help='Specifies minimum number of missing coverage lines needed to report on file') ]

    def report_path(self, path, coverage_data):
        if self.minimum_missing is None:
            return True
        return coverage_data.get_missing_lines_for_path(path) >= self.minimum_missing



import os
import shutil

from coverage_reporter.plugins import Plugin, Option

class AnnotateReporter(Plugin):
    name = 'annotate'

    options = [Option('annotate', 'boolean', help="Write coverage annotations to disk, as place specified by --annotate-dir"),
               Option('annotate_dir', 'string', default='annotate', help="annotation directory, default 'annotate'")]

    def remove_old_annotations(self, annotate_dir):
        if os.path.exists(annotate_dir):
            for root, dirs, files in os.walk(annotate_dir, topdown=False):
                dirs = [ x for x in dirs if os.path.exists(os.path.join(root, x)) ]
                to_remove = [ x for x in files if x.endswith(',cover') ]
                to_keep = [ x for x in files if not x in to_remove ]
                for x in to_remove:
                    os.remove(os.path.join(root, x))
                if not (dirs or to_keep):
                    os.rmdir(root)

    def report(self, coverage_data, path_list):
        annotate_dir = self.annotate_dir
        self.remove_old_annotations(annotate_dir)
        if not os.path.exists(annotate_dir):
            os.makedirs(annotate_dir)

        curdir = os.getcwd()
        coverage = coverage_data.get_lines()
        for path in path_list:
            (total_lines, covered_lines) = coverage_data.get_lines_for_path(path)
            if path.startswith(curdir):
                annotate_path = annotate_dir + path[len(curdir):]
            else:
                annotate_path = annotate_dir + path
            annotate_path += ',cover'
            source = open(path, 'r')
            dir = os.path.dirname(annotate_path)
            if not os.path.exists(dir):
                os.makedirs(dir)
            dest = open(annotate_path, 'w')
            try:
                self.annotate_file(source, dest, total_lines, covered_lines)
            finally:
                source.close()
                dest.close()

    def annotate_file(self, source, dest, total_lines, covered_lines):
        for idx, line in enumerate(source):
            lineno = idx + 1
            if lineno not in total_lines:
                if line.strip():
                    mark = '  '
                else:
                    mark = '  '
            elif lineno not in covered_lines:
                mark = '! '
            else:
                mark = '  '
            dest.write(mark)
            dest.write(line)



import os
import re


def filter_paths(path_list, covered_paths, path_filter):
    if not path_list:
        # No paths were specified by the user, so we only report on covered files.
        path_list = covered_paths
        for path in path_list:
            if path_filter.filter(path):
                yield path
    else:
        for x in filter_specified_paths(path_list, covered_paths, path_filter):
            yield x

def filter_specified_paths(path_list, covered_paths, path_filter):
    """
    These paths were specified at the command line.  Any specifically listed file automatically gets a pass, overriding filters.  
    Any listed directories are recursed and their files are filtered as normal.
    """
    for specified_path in path_list:
        if os.path.isdir(specified_path):
            for root, dirs, filenames in os.walk(specified_path):
                for filename in filenames:
                    full_path = os.path.realpath(os.path.join(root, filename))
                    if path_filter.filter(full_path):
                        yield full_path        
        else:
            yield os.path.realpath(specified_path)



import os

from coverage_reporter.plugins import Plugin, Option

class SummarizeReporter(Plugin):
    name = 'summarize'

    options = [Option('summarize', 'boolean', help="Display summary of coverage information to screen")]

    def report(self, coverage_data, path_list):
        curdir = os.path.realpath(os.getcwd())

        path_totals, final_totals = coverage_data.get_totals()

        new_paths = []
        for path in path_list:
            if path.startswith(curdir):
                new_path = path[len(curdir)+1:]
                path_totals[new_path] = path_totals[path]
                new_paths.append(new_path)
            else:
                new_paths.append(path)

        paths = sorted(x for x in new_paths if path_totals[x]['lines' ])

        if paths:
            longest_path = max([ len(x) for x in paths ])
        else:
            longest_path = 0
        title = '%-*s%8s%8s%8s%8s' % (longest_path, 'Name', 'Stmts',
                                     'Exec', 'Miss', 'Cover')

        print title
        print '-' * len(title)
        for path in paths:
            print '%-*s%8s%8s%8s%8.2f' % (longest_path, path, path_totals[path]['lines'],
                                       path_totals[path]['covered'], path_totals[path]['missing'], path_totals[path]['percent'])
        print '-' * len(title)
        total_lines, total_covered, total_pct = final_totals
        total_missed = total_lines - total_covered
        print '%-*s%8s%8s%8s%8.2f' % (longest_path, 'TOTAL', total_lines,
                                     total_covered, total_missed, total_pct)



"""
Framework for parsing, munging and reporting coverage information.

Build to be extendable by plugins, if anyone should so desire.

Example commands

coverage-reporter --figleaf --annotate --exclude '.*/migrations/.*' <path> [<path>]
coverage-reporter --coverage --patch=stdin --annotate <path> 

TODO - config file parsing fixups.  Have command line way of enabling/disabling plugins.
"""
import sys

from coverage_reporter import config
from coverage_reporter import errors
from coverage_reporter import data

def main(argv):

    cfg = config.CoverageReporterConfig()
    cfg.load_plugins()

    path_list = cfg.parse_options(argv[1:])
    cfg.initialize_plugins()

    collectors = cfg.get_collectors()
    reporters = cfg.get_reporters()

    if not collectors:
        raise errors.CoverageReporterError('Please specify a source for coverage information')

    if not reporters:
        raise errors.CoverageReporterError('Please specify at least one form of reporting.')

    coverage_data = data.CoverageData()
    cover_filter = cfg.get_coverage_filter()
    for collector in cfg.get_collectors():
        new_coverage_data = collector.collect(path_list, cover_filter)
        coverage_data.merge(new_coverage_data)

    for filter in cfg.get_filters():
        coverage_data = filter.filter(coverage_data)
    print "reporting data"

    reporter_filter = cfg.get_reporter_filter()

    all_paths = coverage_data.get_paths() 
    matched_paths = list(reporter_filter.filter_all(all_paths, coverage_data))

    num_skipped_paths = len(all_paths) - len(matched_paths)
    if num_skipped_paths:
        print "Not reporting on %s files due to filters" % (num_skipped_paths,)

    for reporter in cfg.get_reporters():
        reporter.report(coverage_data, matched_paths)

if __name__ == '__main__':
    main(sys.argv)



# Based off of figleaf's CoverageData object,
# Though currently doesn't have support for figleaf sections or
# coverage.py arcs.
import os

class CoverageData(object):
    """
    In general, do not pickle this object; it's simpler and more
    straightforward to just pass the basic Python objects around
    (e.g. CoverageData.covered, a set, and CoverageData.sections, a
    dictionary of sets).
    """
    def __init__(self, lines=None, covered=None):
        if lines is None:
            lines = {}
        if covered is None:
            covered = {}
        self.covered = covered
        self.lines = lines
        self.exclude = []

    def get_covered(self, section_name=None):
        if not section_name:
            return dict(self.covered)
        else:
            d = CoverageData()
            d.update_coverage(self.covered)
            return d.get_covered()

    def get_lines(self):
        line_info = {}
        for path, lines in self.lines.iteritems():
            covered_lines = self.covered.get(path, set())
            line_info[path] = (lines, covered_lines)
        return line_info
    
    def get_lines_for_path(self, path):
        return (self.lines.get(path, set()), self.covered.get(path, set()))

    def get_paths(self):
        return self.lines.keys()

    def get_totals(self):
        report_info = {}
        total_covered = 0
        total_lines = 0
        for path, lines in self.lines.iteritems():
            covered_lines = self.covered.get(path, set())
            covered_lines &= set(lines)
            num_lines = len(lines)
            num_covered = len(covered_lines)
            if num_lines:
                percent_covered = (num_covered * 100)/float(num_lines)
            else:
                percent_covered = 0.0
            report_info[path] = { 'lines' : num_lines, 
                                  'covered' : num_covered, 
                                  'percent' : percent_covered,
                                  'missing' : num_lines - num_covered }
            total_covered += num_covered
            total_lines += num_lines
        if not total_lines:
            total_percent = 0.0
        else:
            total_percent = (total_covered * 100)/float(total_lines)
        return report_info, (total_lines, total_covered, total_percent)

    def get_missing_lines_for_path(self, path):
        num_lines = len(self.lines.get(path, []))
        covered_lines = len(self.covered.get(path, []))
        return num_lines - covered_lines

    def _update_coverage(self, coverage1, coverage2):
        for filename, lines in coverage2.items():
            filename = os.path.realpath(os.path.abspath(filename))
            if filename in coverage1:
                coverage1[filename].update(lines)
            else:
                coverage1[filename] = set(lines)

    def update_coverage(self, coverage_dict):
        self._update_coverage(self.covered, coverage_dict)

    def update_lines(self, line_dict):
        self.lines.update(line_dict)

    def merge(self, other):
        self.lines.update(other.lines)
        self.update_coverage(other.covered)

    def __and__(self, other):
        if isinstance(other, CoverageData):
            coverage_dict = other.data
        else:
            coverage_dict = other

        final_lines = {}
        for path, lines in coverage_dict.iteritems():
            final_lines[path] = set(self.lines.get(path, set())) & set(lines)
        return CoverageData(lines=final_lines, covered=self.covered.copy())




try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
    
Version = "0.01"

setup(name = "coverage-reporter",
      version = Version,
      description = "Coverage reporting tool",
      long_description="Allows more complicated reporting of information from figleaf and other coverage tools",
      author = "David Christian",
      author_email = "david.chrsitian@gmail.com",
      url = "http://github.org/dugan/coverage-reporter/",
      packages = [ 'coverage_reporter', 'coverage_reporter.filters', 'coverage_reporter.collectors', 'coverage_reporter.reports' ],
      license = 'MIT',
      scripts = ['scripts/coverage-reporter'],
      platforms = 'Posix; MacOS X; Windows',
      classifiers = [ 'Intended Audience :: Developers',
                      'License :: OSI Approved :: MIT License',
                      'Operating System :: OS Independent',
                      ],
      )



import re

from coverage_reporter.plugins import Plugin, Option

class ExcludeFilter(Plugin):
    """
    Exclude filter.  Filters like foo.py match files named foo.py.  Filters like bar/ match everything under directory bar.
    Full filters should specify ^ and $.
    """
    name = 'exclude'
    options = [ Option('exclude', 'list', help='Exclude paths from coverage reporting.  Can be specified more than once, takes partial regexps.') ]

    def initialize(self):
        self.exclude_res = [ get_exclude_regexp(x) for x in self.exclude ]

    def cover_path(self, path):
        for exclude_re in self.exclude_res:
            if re.match(exclude_re, path):
                return False
        return True

def get_exclude_regexp(exclude_path):
    exclude_re = exclude_path

    if exclude_re.endswith('$'):
        pass
    elif exclude_re.endswith('/'):
        # re's that end with a / are assumed to mean everything in that directory.
        exclude_re += '.*$'
    else:
        exclude_re += '$'

    if exclude_re.startswith('^'):
        pass
    elif exclude_re.startswith('/'):
        exclude_re = '^.*' + exclude_re
    else:
        exclude_re = '^.*/' + exclude_re
    return exclude_re



import os
import sys

from coverage_reporter.plugins import Plugin, Option

class FilterByPatch(Plugin):
    name = 'patch'
    options = [ Option('patch', 'string', help='Filter coverage reports by patch at PATCH.  Can be file or stdin'),
                Option('patch_level', 'int', help='Patch level, as usually specified to patch command'), ]

    def filter(self, coverage_data):
        return filter_by_patch(coverage_data, self.patch, self.patch_level)

def filter_by_patch(coverage_data, patch_path, patch_level):
    if not os.path.exists(patch_path) and patch_path.lower() == 'stdin':
        patch_file = sys.stdin
    else:
        patch_file = open(patch_path)
    lines = _get_lines_from_patch(patch_file, patch_level)
    coverage_data &= lines
    return coverage_data

def _find_patch_file(path, level):
    path = os.path.sep.join(path.split(os.path.sep)[level:])
    if not os.path.exists(path):
        sys.stderr.write("Could not find file %r with patch level %s - maybe wrong patch level?  Specify -p\n" % (path, level))
        return None
    return os.path.realpath(path)

def _get_lines_from_patch(patch_file, patch_level):
    file_dict = {}
    new_file = None
    for line in patch_file:
        # strip 1 \n from the end.
        line = line[:-1]
        if line.startswith('+++'):
            # place marker - name of file
            # +++ <filename>
            new_file = line.split(None, 1)[1]
            new_file = line.split('\t', 1)[0]
            new_file = _find_patch_file(new_file, patch_level)
            if new_file:
                file_dict.setdefault(new_file, [])
        elif not new_file:
            continue
        elif line.startswith('---'):
            # --- <old filename>
            # place marker - name of old file
            continue
        elif line.startswith('@@'):
            # @@ -old_lineno,old_end +new_lineno,new_end @@ <extra info>
            # place marker - we want to start counting lines at new_lineno
            line = line[line.find('+')+1:]
            line = line[:line.find(',')]
            cur_line = int(line)
        elif line.startswith('+'):
            # added line
            file_dict[new_file].append(cur_line)
            cur_line += 1
        elif line.startswith('-'):
            # we can't cover removed lines.
            continue
        else:
            # unchanged line
            cur_line += 1
    return file_dict



"""
Configuration for the coverage-reporter tool.  

Somewhat vestigial, as it turns out the coverage-reporter tool doesn't have any options, just plugins with options.
It will likely be merged with the plugin manager.
"""
import ConfigParser
import itertools
import optparse
import os
import shlex

from coverage_reporter.plugins import PluginManager
from coverage_reporter.errors import ConfigError


# Default set of plugins.  Need to figure out some way to add plugins without having to re-specify all of these,
# as you probably really want these.  Perhaps move to loading plugins from a set of directories, and only require
# hard-coding the submodule name?
DEFAULT_PLUGINS = ['coverage_reporter.collectors.figleaf_collector.FigleafCollector',
                   'coverage_reporter.collectors.coverage_collector.CoveragePyCollector',
                   'coverage_reporter.filters.patch.FilterByPatch',
                   'coverage_reporter.filters.exclude.ExcludeFilter',
                   'coverage_reporter.filters.minimum.MinimumMissingFilter',
                   'coverage_reporter.reports.summarize.SummarizeReporter',
                   'coverage_reporter.reports.annotate.AnnotateReporter',
                   ]


class CoverageReporterConfig(object):
    def __init__(self):
        self.plugin_manager = PluginManager()

    def load_plugins(self):
        cfg = ConfigParser.RawConfigParser()
        cfg.read(['/etc/coverage_reporter', os.path.expanduser('~/.coverage_reporter'), '.coverage_reporter'])
        section = ConfigSection(cfg, 'coverage_reporter')
        plugin_classes = section.get_list('plugins', [])
        if not plugin_classes:
            plugin_classes = DEFAULT_PLUGINS

        def get_section(name):
            return ConfigSection(cfg, 'coverage_reporter')
        self.plugin_manager.load_plugins(get_section, plugin_classes)
        self.plugins_loaded = True

    def get_option_parser(self):
        parser = optparse.OptionParser()
        self.plugin_manager.call_method('add_options', parser)
        return parser

    def parse_options(self, args):
        parser = self.get_option_parser()
        options, args = parser.parse_args(args)
        self.plugin_manager.call_method('parse_options', options)
        return args

    def initialize_plugins(self):
        self.plugin_manager.call_method('initialize')

    def get_collectors(self):
        return self.plugin_manager.get_collectors()

    def get_filters(self):
        return self.plugin_manager.get_filters()

    def get_coverage_filter(self):
        return self.plugin_manager.get_coverage_filter()

    def get_reporter_filter(self):
        return self.plugin_manager.get_reporter_filter()

    def get_reporters(self):
        return self.plugin_manager.get_reporters()

class ConfigSection(object):
    """
    Very simply wrapper around pretty poor ConfigParser interface.  Sets the current section name so it is only specified once, and then 
    provides a number of safe methods that return None when no element is available.
    """

    def __init__(self, cfg, name):
        self.cfg = cfg
        self.name = name
        if not cfg.has_section(name):
            cfg.add_section(name)

    def get(self, key, default=None):
        if self.cfg.has_option(self.name, key):
            return self.cfg.get(self.name, key)
        return default

    def set(self, key, value):
        return self.cfg.set(self.name, key, str(value))

    def set_list(self, key, value):
        return self.cfg.set(self.name, key, value)

    def get_int(self, key, default=None):
        value = self.get(key, None)
        if value is None:
            return default
        return int(value)

    def get_list(self, item, default=None):
        value = self.get(item, default)
        if value is default:
            return value
        lines = [ line.strip() for line in value.splitlines() ]
        return list(itertools.chain(*[ shlex.split(line) for line in lines if line and not line.startswith('#') ]))
